/* Copyright(c) 1994  Danny Sadinoff
  See section COPYING for conditions for redistribution
*/
#ifndef __MYERROR__
#define __MYERROR__

#include <stdio.h>

void die( const char *, const char * );
void warn( const char *, const char * );

#endif
